﻿using FLY.Thick.Blowing.Client;
using FLY.Thick.Blowing.IService;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using FLY.Thick.Base.UI;
using GalaSoft.MvvmLight.Command;

namespace FLY.Thick.Blowing.UI.Fix.Client
{
    class PgBlowingVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        #region 属性
        #region 参数

        /// <summary>
        /// 离开限位 到 撞下一个限位 的 旋转架转动总角度 单位°
        /// </summary>
        public double RAngle { get; set; }

        /// <summary>
        /// 人字架 旋转1周 设置 时间。 
        /// 刚开机时，RenZiJiaPeriod = DefaultRPeriod；
        /// 异常时，RenZiJiaPeriod = DefaultRPeriod；
        /// </summary>
        public TimeSpan DefaultRPeriod { get; set; }

        /// <summary>
        /// 加速减速时间，  默认1s  加速减速时间 * 2 +正反转冷却时间 = 撞限位 到 离开限位 的时间
        /// </summary>
        public TimeSpan AccDecTime { get; set; }

        /// <summary>
        /// 信号0 撞2次。 这个能自动设置。 两个撞同一信号 间隔30秒内，算一个换向信号。 
        /// 当为2次撞时：  StartTime = 第1次撞的StartTime， EndTime = 第2次撞的EndTime
        /// </summary>
        public bool IsSign0Double { get; set; }

        /// <summary>
        /// 信号1 撞2次。 这个能自动设置。 两个撞同一信号 间隔30秒内，算一个换向信号。 
        /// 当为2次撞时：  StartTime = 第1次撞的StartTime， EndTime = 第2次撞的EndTime
        /// </summary>
        public bool IsSign1Double { get; set; }

        /// <summary>
        /// 信号模式， 短信号模式（只撞一下），长信号模式（只要是转着，信号就长亮，直到换向）
        /// </summary>
        public BlowingSignType SignType { get; set; }

        /// <summary>
        /// 撞限位 到 离开限位 的时间， 需要初始值，以后测量出来的
        /// </summary>
        public TimeSpan LimitSignTime { get; set; }

        /// <summary>
        /// 人字架到测厚仪膜长 单位m
        /// 当旋转架为立式时， 膜长为人字架复位时，的最短膜距离
        /// </summary>
        public double FilmLength { get; set; }

        /// <summary>
        /// 辊周长，单位mm
        /// </summary>
        public double RollPerimeter { get; set; }

        /// <summary>
        /// 转向信号100ms 滤波
        /// </summary>
        public bool IsSignFilter { get; set; }

        /// <summary>
        /// 缺少信号模式
        /// 正反信号线接得不好，有时候会丢失了。 
        /// 当前的旋转时间是上一次的1.1倍时，在以前的位置增加信号！！！！
        /// 当有连续两次相同的信号，且信号差刚好是旋转时间的1.9~2.1倍时，证明真的是缺少信号
        /// </summary>
        public bool IsLackSignMode { get; set; }

        /// <summary>
        /// 旋转架为立式，测厚仪安装在二牵引前面，膜距离 与 旋转角度 一起变化， 旋转角度越大 膜距离越大
        /// </summary>
        public bool Is3D { get; set; }



        #endregion
        #region 分区设定
        /// <summary>
        /// 加热通道数
        /// </summary>
        public int ChannelCnt { get; set; }

        /// <summary>
        /// 分区数/加热通道数
        /// </summary>
        public int BPC { get; set; }

        /// <summary>
        /// 人字架总分区数，从1开始数
        /// </summary>
        public int NBolts => ChannelCnt * BPC;

        /// <summary>
        /// 人字架复位时, 探头对应的分区位， 也就是 0° 对应的分区位
        /// </summary>
        public int OrgBoltNo { get; set; }

        /// <summary>
        /// 使用分区表
        /// </summary>
        public bool IsUsedMap { get; set; }

        /// <summary>
        /// 分区表
        /// </summary>
        public ObservableCollection<BoltMapCell> BoltMap { get; } = new ObservableCollection<BoltMapCell>();
        #endregion
        /// <summary>
        /// 数据加载中......
        /// </summary>
        public bool IsBufListLoading { get; set; }
        /// <summary>
        /// 换向信号列表
        /// </summary>
        public ObservableCollection<SignData> SignList { get; } = new ObservableCollection<SignData>();

        #endregion

        #region Command
        public RelayCommand ApplyCmd { get; }
        public RelayCommand DownloadCmd { get; }
        #endregion
        public IBlowingFixService renZiJiaService { get; private set; }
        public IBlowingDetectService bDetect { get; private set; }

        public PgBlowingVm()
        {
            ApplyCmd = new RelayCommand(Apply);
            DownloadCmd = new RelayCommand(Download);
        }



        public void Init(
            IBlowingFixService blowingFixService,
            IBlowingDetectService blowingDetectService)
        {
            renZiJiaService = blowingFixService;
            bDetect = blowingDetectService;

            Misc.BindingOperations.SetBinding(bDetect, "RAngle", this, "RAngle");
            Misc.BindingOperations.SetBinding(bDetect, "DefaultRPeriod", this, "DefaultRPeriod");
            Misc.BindingOperations.SetBinding(bDetect, "AccDecTime", this, "AccDecTime");
            Misc.BindingOperations.SetBinding(bDetect, "IsSign0Double", this, "IsSign0Double");
            Misc.BindingOperations.SetBinding(bDetect, "IsSign1Double", this, "IsSign1Double");
            Misc.BindingOperations.SetBinding(bDetect, "SignType", this, "SignType");
            Misc.BindingOperations.SetBinding(bDetect, "LimitSignTime", this, "LimitSignTime");
            Misc.BindingOperations.SetBinding(bDetect, "FilmLength", this, "FilmLength");
            Misc.BindingOperations.SetBinding(bDetect, "RollPerimeter", this, "RollPerimeter");
            Misc.BindingOperations.SetBinding(bDetect, "IsSignFilter", this, "IsSignFilter");
            Misc.BindingOperations.SetBinding(bDetect, "IsLackSignMode", this, "IsLackSignMode");
            Misc.BindingOperations.SetBinding(bDetect, "Is3D", this, "Is3D");

            Misc.BindingOperations.SetBinding(renZiJiaService, "ChannelCnt", this, "ChannelCnt");
            Misc.BindingOperations.SetBinding(renZiJiaService, "BPC", this, "BPC");
            Misc.BindingOperations.SetBinding(renZiJiaService, "OrgBoltNo", this, "OrgBoltNo");
            Misc.BindingOperations.SetBinding(renZiJiaService, "IsUsedMap", this, "IsUsedMap");
            Misc.BindingOperations.SetBinding(renZiJiaService, "Map", ()=> {
                BoltMap.Clear();
                foreach (BoltMapCell m in renZiJiaService.Map)
                {
                    BoltMap.Add(m);
                }
            });

        }

        void Download()
        {
            IsBufListLoading = true;
            bDetect.GetSignList(
                (AsyncState, retData) =>
                {
                    GetSignListReponse reponse = retData as GetSignListReponse;
                    //reponse.firstbm

                    IsBufListLoading = false;


                    SignList.Clear();
                    //反转顺序
                    if (reponse.datas != null)
                    {
                        for (int i = 0; i < reponse.datas.Count(); i++)
                        {
                            int idx = reponse.datas.Count() - 1 - i;
                            SignList.Add(reponse.datas[idx]);
                        }
                    }

                }, null);
        }

        void Apply()
        {
            if (!WdPassword.Authorize("Blowing"))
                return;

            string message;
            bool isvalid = CheckValid(out message);
            if (!isvalid)
            {
                //异常出错
                FLY.ControlLibrary.Window_WarningTip.Show(
                    "参数出错",
                    message + "异常");
            }
            else
            {
                //TODO
                //设置到服务
                bDetect.RAngle = this.RAngle;
                bDetect.DefaultRPeriod=this.DefaultRPeriod;
                bDetect.AccDecTime=this.AccDecTime;
                bDetect.IsSign0Double=this.IsSign0Double;
                bDetect.IsSign1Double=this.IsSign1Double;
                bDetect.SignType=this.SignType;
                bDetect.LimitSignTime=this.LimitSignTime;
                bDetect.FilmLength=this.FilmLength;
                bDetect.RollPerimeter=this.RollPerimeter;
                bDetect.IsSignFilter=this.IsSignFilter;
                bDetect.IsLackSignMode=this.IsLackSignMode;
                bDetect.Is3D=this.Is3D;
                bDetect.Apply();

                renZiJiaService.ChannelCnt=this.ChannelCnt;
                renZiJiaService.BPC=this.BPC;
                renZiJiaService.OrgBoltNo=this.OrgBoltNo;
                renZiJiaService.IsUsedMap=this.IsUsedMap;
                renZiJiaService.Apply();

                FLY.ControlLibrary.Window_Tip.Show("应用成功",
                    null,
                    TimeSpan.FromSeconds(2));
            }
            
        }

        bool CheckValid(out string message)
        {
            //检查参数是否合法
            if (this.RAngle < 100 || this.RAngle > 360)
            {
                message = "旋转总角度";
                return false;
            }

            if ((this.DefaultRPeriod.TotalMinutes < 3) || (this.DefaultRPeriod.TotalMinutes > 25))
            {
                message = "旋转时间";
                return false;
            }

            if ((this.RollPerimeter < 100) || (this.RollPerimeter > 1000))
            {
                message = "辊周长";
                return false;
            }
            if ((this.FilmLength < 0) || (this.FilmLength > 100))
            {
                message = "膜距离";
                return false;
            }

            if (this.AccDecTime.TotalMilliseconds > 5000)
            {
                message = "加减速时间";
                return false;
            }

            if (this.LimitSignTime.TotalSeconds > 30)
            {
                message = "撞信号总时间";
                return false;
            }


            if (this.ChannelCnt < 20 || this.ChannelCnt > 160)
            {
                message = "加热通道数";
                return false;
            }
            if (this.BPC < 1 || this.BPC > 4)
            {
                message = "分区数/加热通道数";
                return false;
            }
            if (this.OrgBoltNo < 1 || this.OrgBoltNo > (this.ChannelCnt * this.BPC))
            {
                message = "复位区号";
                return false;
            }
            message = "成功";
            return true;
        }
    }




}
