﻿
using FLY.ControlLibrary;
using FLY.Thick.Base.IService;
using FLY.Thick.Blowing.IService;
using FLY.Thick.BulkDataModule;
using MultiLayout;
using MultiLayout.UiModule;
using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using Unity;

namespace FLY.Thick.Blowing.UI.Fix.Client.UiModule
{
    /// <summary>
    /// ScanGraphCircular.xaml 的交互逻辑
    /// </summary>
    public partial class ScanGraphCircular : UserControl
    {

        ScanGraphCircularParam graphparam;
        BulkDataServiceClientAdv bulkDataServiceClientAdv;
        IDynAreaService dynAreaService;
        IBlowingService blowingService;
        public ScanGraphCircular()
        {
            InitializeComponent();
        }

        [InjectionMethod]
        public void Init(int id,
            IDynAreaService dynAreaService,
            BulkDataServiceClientAdv bulkDataServiceClientAdv,
            IBlowingService blowingService)
        {
            this.dynAreaService = dynAreaService;
            this.bulkDataServiceClientAdv = bulkDataServiceClientAdv;
            this.blowingService = blowingService;

            //查找参数
            graphparam = ScanGraphCircularParams.Current.Items.Find(p => p.ID == id);
            if (graphparam == null)
            {
                graphparam = new ScanGraphCircularParam() { ID = id };
                ScanGraphCircularParams.Current.Items.Add(graphparam);
                ScanGraphCircularParams.Current.Save();
            }


            GraphScanCircular graph = graphscan;

            graph.DataContext = graphparam;
            graph.SetBinding(GraphScanCircular.TitleProperty, new Binding("Title"));
            graph.SetBinding(Grid.ColumnProperty, new Binding("Column"));
            graph.SetBinding(Grid.RowProperty, new Binding("Row"));
            graph.SetBinding(Grid.ColumnSpanProperty, new Binding("ColumnSpan"));
            graph.SetBinding(Grid.RowSpanProperty, new Binding("RowSpan"));
            graph.SetBinding(GraphScanCircular.BoltNoWithManual1stProperty, new Binding("BoltNoWithManual1st"));
            graph.SetBinding(GraphScanCircular.ManualCntProperty, new Binding("ManualCnt"));
            graph.SetBinding(GraphScanCircular.MPIsRightProperty, new Binding("MPIsRight"));

            //TODO
            graph.SetBinding(GraphScanCircular.BoltCntProperty, new Binding("NBolts") { Source = blowingService });
            graph.SetBinding(GraphScanCircular.OrgBoltNoProperty, new Binding("OrgBoltNo") { Source = blowingService });

            graph.SetBinding(GraphScanCircular.TargetProperty, new Binding("Target") { Source = dynAreaService.DynArea });
            graph.SetBinding(GraphScanCircular.AlarmProperty, new Binding("Alarm") { Source = dynAreaService.DynArea });




            //bulkdata 数据绑定！！！
            BulkDataServiceClientAdv.BulkDataGraphFrame bulkdata_client = new BulkDataServiceClientAdv.BulkDataGraphFrame();
            //graphparam 绑定到 bulkdataclient
            Misc.BindingOperations.SetBinding(graphparam, "BM", bulkdata_client, "BM");
            Misc.BindingOperations.SetBinding(graphparam, "Mix", bulkdata_client, "Mix");
            graph.SetBinding(GraphScanCircular.Title2Property, new Binding("Info") { Source = bulkdata_client });
            graph.DataSource = bulkdata_client.Frame;


            graph.ShowSettingDialogEvent += new EventHandler(delegate (object sender, EventArgs e)
            {
                ScanGraphCircularConfig window = new ScanGraphCircularConfig();
                window.Init((sender as GraphScanCircular).DataContext as ScanGraphCircularParam);
                window.Owner = FLY.ControlLibrary.COMMON.GetWindow(graph);
                window.ShowDialog();
            });

            bulkDataServiceClientAdv.Add(bulkdata_client);
        }
    }
    public class UiModule2_ScanGraphCircular : MultiLayout.UiModule.IUiModule2
    {
        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title => "环型图";
        public ComponentType Type => ComponentType.Graph;
        public bool IsUnique => false;

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id, IUnityContainer container)
        {
            return container.Resolve<ScanGraphCircular>(new Unity.Resolution.ParameterOverride("id", id));
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <returns></returns>
        public FrameworkElement GetThumbnail()
        {
            return new System.Windows.Controls.Grid();
        }


        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
            ScanGraphCircularParams.Current.MatchParam(IDs);
        }
    }
    public class ScanGraphCircularParam : UIModuleParam
    {
        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; } = "环形扫描图";

        /// <summary>
        /// 混合数
        /// </summary>
        public int Mix { get; set; } = 1;


        /// <summary>
        /// 记录点
        /// </summary>
        public int BM { get; set; } = -1;


        /// <summary>
        /// 模头手动调节的螺丝对应的分区号
        /// </summary>
        public int BoltNoWithManual1st { get; set; } = 1;


        /// <summary>
        /// 模头手动调节的螺丝个数
        /// </summary>
        public int ManualCnt { get; set; } = 8;


        /// <summary>
        /// 探头在右边
        /// </summary>
        public bool MPIsRight { get; set; } = true;
    }

    /// <summary>
    /// 全部扫描图的参数
    /// </summary>
    public class ScanGraphCircularParams : UIModuleParams<ScanGraphCircularParam>
    {
        static ScanGraphCircularParams()
        {
        }
        public ScanGraphCircularParams()
        {
            string path = System.IO.Path.Combine(FlyLayout.BasePath, "scangraphcircular.json");
            FilePath = path;
        }

        static ScanGraphCircularParams current = null;
        public static ScanGraphCircularParams Current
        {
            get
            {
                if (current == null)
                {
                    current = new ScanGraphCircularParams();
                    current.Load();
                }
                return current;
            }
        }
    }
}
