﻿using FLY.ControlLibrary;
using FLY.Thick.Base.IService;
using FLY.Thick.BulkDataModule;
using MultiLayout;
using MultiLayout.UiModule;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using Unity;

namespace FLY.Thick.Blowing.UI.Fix.Client.UiModule
{
    /// <summary>
    /// TrendGraph.xaml 的交互逻辑
    /// </summary>
    public partial class TrendGraph : UserControl
    {

        TrendGraphItemParam graphparam;
        BulkDataServiceClientAdv bulkDataServiceClientAdv;
        IDynAreaService dynAreaService;

        public TrendGraph()
        {
            InitializeComponent();
        }

        [InjectionMethod]
        public void Init(int id,
            IDynAreaService dynAreaService,
            BulkDataServiceClientAdv bulkDataServiceClientAdv)
        {

            this.dynAreaService = dynAreaService;
            this.bulkDataServiceClientAdv = bulkDataServiceClientAdv;


            graphparam = TrendGraphParams.Current.Items.Find(p => p.ID == id);
            if (graphparam == null)
            {
                graphparam = new TrendGraphItemParam() { ID = id };
                TrendGraphParams.Current.Items.Add(graphparam);
                TrendGraphParams.Current.Save();
            }

            GraphRange graph = graphrange;
            graph.DataContext = graphparam;
            //graph.SetValue(GraphRange.MarginProperty, new Thickness(5));

            graph.SetBinding(Grid.ColumnProperty, new Binding("Column"));
            graph.SetBinding(Grid.RowProperty, new Binding("Row"));
            graph.SetBinding(Grid.ColumnSpanProperty, new Binding("ColumnSpan"));
            graph.SetBinding(Grid.RowSpanProperty, new Binding("RowSpan"));

            graph.SetBinding(GraphRange.DataCntProperty, new Binding("Len"));
            graph.SetBinding(GraphRange.TitleProperty, new Binding("Title"));
            graph.SetBinding(GraphRange.IsAutoTargetProperty, new Binding("IsAutoTarget"));
            graph.SetBinding(GraphRange.IsPercentProperty, new Binding("IsPercent"));
            graph.SetBinding(GraphRange.YRangePercentProperty, new Binding("YRangePercent"));
            graph.SetBinding(GraphRange.TypeProperty, new Binding("Type"));
            graph.SetBinding(GraphRange.XAxisTypeProperty, new Binding("XType"));

            graph.SetBinding(GraphRange.TargetProperty, new Binding("Target") { Source = dynAreaService.DynArea });
            graph.SetBinding(GraphRange.AlarmProperty, new Binding("Alarm") { Source = dynAreaService.DynArea });


            BulkDataServiceClientAdv.BulkDataGraphTrend bulkdata_client = new BulkDataServiceClientAdv.BulkDataGraphTrend();

            Misc.BindingOperations.SetBinding(graphparam, "BM", bulkdata_client, "BM", Misc.BindingOperations.BindingMode.OneWay);
            Misc.BindingOperations.SetBinding(graphparam, "Mix", bulkdata_client, "Mix", Misc.BindingOperations.BindingMode.OneWay);
            Misc.BindingOperations.SetBinding(graphparam, "Len", bulkdata_client, "Len", Misc.BindingOperations.BindingMode.OneWay);

            Misc.BindingOperations.SetBinding(graphparam, new string[] { "BeginNo", "IsAll" }, delegate ()
            {
                if (graphparam.IsAll)
                    bulkdata_client.FromBoltNo = Misc.MyBase.NULL_VALUE;
                else
                    bulkdata_client.FromBoltNo = graphparam.BeginNo;
            });

            Misc.BindingOperations.SetBinding(graphparam, "EndNo", bulkdata_client, "ToBoltNo", Misc.BindingOperations.BindingMode.OneWay);

            GraphRangeDataValueChanged vc = new GraphRangeDataValueChanged();
            vc.Init(bulkdata_client.Frame, graph);

            graph.SetBinding(GraphRange.Title2Property, new Binding("Info") { Source = bulkdata_client });


            graph.ShowSettingDialogEvent += new EventHandler(delegate (object sender, EventArgs e)
            {
                TrendGraphConfig w = new TrendGraphConfig();

                TrendGraphItemParam graphparam_src = (sender as GraphRange).DataContext as TrendGraphItemParam;

                w.DataContext = graphparam_src;
                w.SaveEvent += (s, ea) =>
                {
                    TrendGraphParams.Current.Save();
                };
                w.Owner = FLY.ControlLibrary.COMMON.GetWindow(graph);
                w.ShowDialog();
            });

            //BulkDataServiceClientAdv mBulkDataClient;
            //if (mBulkDataClients.ContainsKey(graphparam.BulkData))
            //    mBulkDataClient = mBulkDataClients[graphparam.BulkData];
            //else
            //    mBulkDataClient = mBulkDataClients.First().Value;

            bulkDataServiceClientAdv.Add(bulkdata_client);
        }


        public class GraphRangeDataValueChanged
        {
            ObservableCollection<GraphRangeData> rangedata = new ObservableCollection<GraphRangeData>();

            public void Init(ObservableCollection<TrendDataCell> Frame, GraphRange graph)
            {
                Frame.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(trenddata_CollectionChanged);
                graph.DataSource = rangedata;
            }
            void trenddata_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
            {
                switch (e.Action)
                {
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Add:
                        foreach (TrendDataCell v in e.NewItems)
                        {
                            rangedata.Add(
                                new GraphRangeData()
                                {
                                    Avg = v.Avg,
                                    Sigma = v.Sigma,
                                    Time = v.Time,
                                    InCtrl = (v.Type == FR_DATA_TYPE.VALID)
                                });
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Reset:
                        {
                            rangedata.Clear();
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Replace:
                        {
                            for (int i = 0; i < e.OldItems.Count; i++)
                            {
                                TrendDataCell v = (TrendDataCell)e.NewItems[i];
                                rangedata[i + e.OldStartingIndex] = new GraphRangeData()
                                {
                                    Avg = v.Avg,
                                    Sigma = v.Sigma,
                                    Time = v.Time,
                                    InCtrl = (v.Type == FR_DATA_TYPE.VALID)
                                };
                            }
                        }
                        break;
                    case System.Collections.Specialized.NotifyCollectionChangedAction.Remove:
                        {
                            for (int i = 0; i < e.OldItems.Count; i++)
                            {
                                rangedata.RemoveAt(i + e.OldStartingIndex);
                            }
                        }
                        break;
                }
            }


        }
    }

    /// <summary>
    /// 趋势图的参数
    /// </summary>
    public class TrendGraphItemParam : UIModuleParam
    {

        /// <summary>
        /// 数据量
        /// </summary>
        public int Len { get; set; } = 120;


        /// <summary>
        /// 统计开始数据序号
        /// </summary>
        public int BeginNo { get; set; }


        /// <summary>
        /// 统计结束数据序号
        /// </summary>
        public int EndNo { get; set; } = 20;


        /// <summary>
        /// 全部数据
        /// </summary>
        public bool IsAll { get; set; } = true;

        /// <summary>
        /// 曲线类型
        /// </summary>
        public FLY.ControlLibrary.GraphRangeType Type { get; set; } = FLY.ControlLibrary.GraphRangeType.Avg;

 
        /// <summary>
        /// X轴类型
        /// </summary>
        public FLY.ControlLibrary.GraphRangeXAxisType XType { get; set; } = FLY.ControlLibrary.GraphRangeXAxisType.Time;

        /// <summary>
        /// 记录点
        /// </summary>
        public int BM { get; set; }


        /// <summary>
        /// Y轴比例
        /// </summary>
        public double YRangePercent { get; set; } = 3;


        /// <summary>
        /// 标题
        /// </summary>
        public string Title { get; set; } = "趋势图";

        /// <summary>
        /// 混合图
        /// </summary>
        public int Mix { get; set; } = 1;


        /// <summary>
        /// 自动目标值
        /// </summary>
        public bool IsAutoTarget { get; set; }


        /// <summary>
        /// % 显示
        /// </summary>
        public bool IsPercent { get; set; }
    }

    /// <summary>
    /// 全部扫描图的参数
    /// </summary>
    public class TrendGraphParams : UIModuleParams<TrendGraphItemParam>
    {

        public TrendGraphParams()
        {
            string path = System.IO.Path.Combine(FlyLayout.BasePath, "trendgraph.json");
            FilePath = path;
        }

        static TrendGraphParams current = null;
        public static TrendGraphParams Current
        {
            get
            {
                if (current == null)
                {
                    current = new TrendGraphParams();
                    current.Load();
                }
                return current;
            }
        }
    }

    /// <summary>
    /// 扫描图控件模块
    /// </summary>
    public class UiModule2_TrendGraph : IUiModule2
    {

        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title => "纵向趋势图";
        public ComponentType Type => ComponentType.Graph;
        public bool IsUnique => false;

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id, IUnityContainer container)
        {
            return container.Resolve<TrendGraph>(new Unity.Resolution.ParameterOverride("id", id));
        }


        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail()
        {
            return new Grid();
        }

        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
            TrendGraphParams.Current.MatchParam(IDs);
        }
    }
}
