﻿using FLY.OBJComponents.Client;
using FLY.OBJComponents.IService;
using FLY.Thick.Base.UI;
using FLY.Weight.Client;
using FLY.Weight.Common;
using FLY.Weight.IService;
using FLY.Weight.Server.Model;
using Misc;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Unity;

namespace FLY.Weight.UI.Client
{
    /// <summary>
    /// Page_RollMixTable.xaml 的交互逻辑
    /// </summary>
    public partial class PgRollMixTable : Page
    {
        IUnityContainer container;
        IWeightSystemService weightSystemService;
        WeighterC mWeighter;
        IBuffer<Lc_Mix> buffer;
        ParamDictionary paramDictionary;

        BufferWindow<Lc_Mix> mWindow;

        public PgRollMixTable()
        {
            InitializeComponent();
            

        }

        [InjectionMethod]
        public void Init(
            IUnityContainer container,
            WeighterC w,
            IWeightSystemService weightSystemService,
            MixBufferCollection mixBufferCollection,
            ParamDictionary paramDictionary)
        {
            this.container = container;
            this.paramDictionary = paramDictionary;
            this.weightSystemService = weightSystemService;
            mWeighter = w;
            int item_idx = this.weightSystemService.Items.IndexOf(w);
            this.buffer = mixBufferCollection.GetRollMix(item_idx);


            //templateRoot.DataContext = w;

            //窗口显示数据条数
            int windowSize = this.paramDictionary.GetValue<int>(ParamDistItemKeys.WindowSize, 30);
            mWindow = new BufferWindow<Lc_Mix>(buffer, windowSize);
            mWindow.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == "Size")
                {
                    this.paramDictionary.SetValue(ParamDistItemKeys.WindowSize, mWindow.Size);
                    this.paramDictionary.Save();
                }
            };


            //界面........................................

            InitDataGrid();

            grid_window.DataContext = mWindow;
        
        }

        void InitDataGrid()
        {
            DataGrid datagrid = gridFlows;
            for (int i = 0; i < mWeighter.Ingredients.Count(); i++)
            {
                // kg
                DataGridTextColumn dgtc = new DataGridTextColumn()
                {
                    Binding = new Binding() { Path = new PropertyPath($"Items[{i}].Disp"), StringFormat = "{0:F1}" }
                };
                StackPanel sp = new StackPanel() { Orientation = Orientation.Horizontal };
                sp.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldHeader") as Style,
                        Text = $"∑{mWeighter.Ingredients[i].Number}#"
                    });
                sp.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldContent_mm") as Style,
                        Text = "kg",
                        Margin = new Thickness(10, 0, 10, 0)
                    });
                dgtc.Header = sp;
                gridFlows.Columns.Add(dgtc);

                // %
                DataGridTextColumn dgtc2 = new DataGridTextColumn()
                {
                    Binding = new Binding() { Path = new PropertyPath($"Items[{i}].PDisp"), StringFormat = "{0:F1}" }
                };
                StackPanel sp2 = new StackPanel() { Orientation = Orientation.Horizontal };
                sp2.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldHeader") as Style,
                        Text = $"∑{mWeighter.Ingredients[i].Number}#"
                    });
                sp2.Children.Add(
                    new TextBlock()
                    {
                        Style = this.FindResource("TextBlockStyle_FieldContent_mm") as Style,
                        Text = "%",
                        Margin = new Thickness(10, 0, 10, 0)
                    });
                dgtc2.Header = sp2;
                dgtc2.CellStyle = this.FindResource("DataGridCellStyle_Percent") as Style;

                gridFlows.Columns.Add(dgtc2);

            }
        }

        private void button_back_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.GoBack();


            mWindow.Dispose();
        }

        private void button_clear_click(object sender, RoutedEventArgs e)
        {
            if (FLY.ControlLibrary.MyMessageBox.Show("确定是否清空全部历史数据?") == true)
            {
                buffer.Reset();

                FLY.ControlLibrary.Window_Tip.Show("成功", "清除完毕!", TimeSpan.FromSeconds(2));
            }
        }

        private void button_prepage_Click(object sender, RoutedEventArgs e)
        {
            mWindow.MovePrePage();
        }

        private void button_nextpage_Click(object sender, RoutedEventArgs e)
        {
            mWindow.MoveNextPage();
        }

        private void button_newest_Click(object sender, RoutedEventArgs e)
        {
            mWindow.MoveNewest();
        }
    }
}
