﻿using FLY.OBJComponents.Client;
using FLY.OBJComponents.IService;
using FLY.Weight.Client;
using FLY.Weight.IService;
using FLY.Weight.Server.Model;
using LiveCharts;
using LiveCharts.Configurations;
using LiveCharts.Wpf;
using MultiLayout;
using MultiLayout.UiModule;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Navigation;
using System.Windows.Threading;
using Unity;

namespace FLY.Weight.UI.Client.UiModule
{
    /// <summary>
    /// FlowGraph.xaml 的交互逻辑
    /// </summary>
    public partial class FlowGraph : UserControl
    {
        IUnityContainer container;
        IWeightSystemService weightSystemService;
        FlowGraphVm flowGraphModelView;
        public FlowGraph()
        {
            InitializeComponent();
        }

       [InjectionMethod]
        public void Init(
            IUnityContainer container,
             IWeightSystemService weightSystemService,
            IBuffer<Lc_Flow> flowList)
        {
            this.container = container;
            this.weightSystemService = weightSystemService;
            flowGraphModelView = new FlowGraphVm();
            flowGraphModelView.Init(weightSystemService, flowList);

            grid_modelview.DataContext = flowGraphModelView;
            grid_bufferwindow.DataContext = flowGraphModelView.bufferWindow;
        }



        private void button_info_Click(object sender, RoutedEventArgs e)
        {
            WdGraphSet w = new WdGraphSet();
            w.Init(weightSystemService);
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            w.ShowDialog();
        }

        private void button_newest_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.Newest();
        }

        private void button_pre_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.Pre();
        }

        private void button_next_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.Next();
        }

        private void button_abc_Click(object sender, RoutedEventArgs e)
        {
            LineSeries series = ((Button)sender).Tag as LineSeries;

            if (series == null) return;

            if (series.Visibility == Visibility.Visible)
                series.Visibility = Visibility.Hidden;
            else
                series.Visibility = Visibility;
        }

        private void button_table_Click(object sender, RoutedEventArgs e)
        {
            PgFlowTable p = new PgFlowTable();
            container.BuildUp(p);

            MultiLayout.FlyLayoutManager.NavigationService.Navigate(p);
        }

        private void button_select_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.SelectABC(((Button)sender).Tag as FlowGraphVm_Item);
        }

        private void button_selectall_Click(object sender, RoutedEventArgs e)
        {
            flowGraphModelView.SelectAll();
        }
    }
    public class FlowGraphVm : INotifyPropertyChanged
    {
        const int MARKNO_UPDATE_Y = 0;
        const int MARKNO_TOVALUES = 1;
        /// <summary>
        /// 当前现在不是最新，而且没有移动动作，20秒自动按 Newest()
        /// </summary>
        DispatcherTimer timer;
        public event PropertyChangedEventHandler PropertyChanged;

        IWeightSystemService weightSystemService;
        IBuffer<Lc_Flow> flowList;
        FlowGraphParams graphParams;

        public BufferWindow<Lc_Flow> bufferWindow;

        /// <summary>
        /// 画面只显示200个数据， 就算设置画面设置显示1000个数，也会整合为200个数据
        /// </summary>
        const int ValuesCap = 200;
        /// <summary>
        /// 自动按保持最新值，总时间
        /// </summary>
        const int AutoKeepNewestTimeCounter = 10;
        #region property
        /// <summary>
        ///自动按保持最新值，剩余时间
        /// </summary>
        public int AutoKeepNewestTimeRemaining { get; set; }


        /// <summary>
        /// 曲线, 是由 bufferWindow.Record 生成的
        /// Values 最大数量只有 ValuesCap
        /// 平均从 bufferWindow.Record提取， 也就是Record有1000个数，也只会间隔抽取200个画出来
        /// </summary>
        public ChartValues<Lc_Flow> Values { get; set; }

        /// <summary>
        /// 曲线值转换器
        /// </summary>
        public CartesianMapper<Lc_Flow> Mapper { get; set; }
        /// <summary>
        /// 曲线颜色
        /// </summary>
        public Brush Stroke { get; set; }

        /// <summary>
        /// X轴格式
        /// </summary>
        public Func<double, string> DateTimeFormatter { get; set; }

        /// <summary>
        /// y轴格式
        /// </summary>
        public Func<double, string> YFormatter { get; set; }

        /// <summary>
        /// 折径目标值
        /// </summary>
        public double AxisYTarget { get; set; }

        /// <summary>
        /// 折径 最大值
        /// </summary>
        public double MaxValue { get; set; }
        /// <summary>
        /// 折径 最小值
        /// </summary>
        public double MinValue { get; set; }

        /// <summary>
        /// 开始时间
        /// </summary>
        public DateTime BeginTime { get; set; }

        /// <summary>
        /// 结束数据
        /// </summary>
        public DateTime EndTime { get; set; }


        public ObservableCollection<FlowGraphVm_Item> Items { get; } = new ObservableCollection<FlowGraphVm_Item>();
        /// <summary>
        /// 总流量
        /// </summary>
        public double TotalFlow { get; set; }
        #endregion

        public FlowGraphVm()
        {

            DateTimeFormatter = value => new DateTime((long)value).ToString("HH:mm:ss");

            YFormatter = value => value.ToString("F1");

            Values = new ChartValues<Lc_Flow>();

        }


        public void Init(
            IWeightSystemService weightSystemService,
            IBuffer<Lc_Flow> flowList
            )
        {
            graphParams = FlowGraphParams.Current;
            this.weightSystemService = weightSystemService;
            this.flowList = flowList;


            bufferWindow = new BufferWindow<Lc_Flow>(flowList, graphParams.WindowSize);

            Misc.BindingOperations.SetBinding(graphParams, "WindowSize", bufferWindow, "Size");

            Misc.BindingOperations.SetBinding(
                this.weightSystemService.Accessory, "TotalFlowSetting", this, "AxisYTarget");

            ToValues();
            bufferWindow.Record.CollectionChanged += //Record_CollectionChanged;
             (s, e) =>
            {
                FObjBase.PollModule.Current.Poll_JustOnce(
                    () =>
                    {
                        ToValues();
                    }, this, MARKNO_TOVALUES);
                ;
            };

            //根据 weightSystemClient.Item创建Series

            Stroke = new SolidColorBrush(Colors.Black);
            Mapper = ConfigurationAll;

            BindingSeries();

            if (this.weightSystemService is WeightSystemServiceClient) {
                (this.weightSystemService as WeightSystemServiceClient).ResetItemsEvent += BindingSeries;
            }



            Update_y();


            this.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == "AxisYTarget")
                    Update_y();
            };

            graphParams.PropertyChanged += (s, e) =>
            {
                if (e.PropertyName == "IsAxisYRangeAuto")
                {
                    Update_y();
                }
                else if (e.PropertyName == "AxisYRange")
                {
                    Update_y();
                }
            };


            //当不是最新数据时，10秒内无操作，自动按 [最新]
            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(1);
            timer.Tick += Timer_Tick;
            bufferWindow.PropertyChanged += BufferWindow_PropertyChanged;

        }
        CartesianMapper<Lc_Flow> ConfigurationAll = Mappers.Xy<Lc_Flow>()
                .X(value => value.Time.Ticks)
                .Y(value =>
                {
                    return value.Total;
                });

        double values_i;

        private void Record_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case System.Collections.Specialized.NotifyCollectionChangedAction.Add:
                    {
                        if (bufferWindow.Size <= ValuesCap)
                        {
                            foreach (var f in e.NewItems)
                            {
                                Values.Add((Lc_Flow)f);
                            }
                        }
                        else
                        {
                            foreach (var f in e.NewItems)
                            {
                                values_i++;
                                if (values_i > bufferWindow.Size / ValuesCap)
                                {
                                    values_i -= bufferWindow.Size / ValuesCap;
                                    Values.Add((Lc_Flow)f);
                                }
                            }
                        }
                        while (Values.Count() > bufferWindow.Size)
                            Values.RemoveAt(0);

                        UpdateAxisYTarget();
                    }
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Reset:
                    {
                        values_i = 0;
                        Values.Clear();
                    }
                    break;
            }
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            AutoKeepNewestTimeRemaining--;
            if (AutoKeepNewestTimeRemaining <= 0)
            {
                AutoKeepNewestTimeRemaining = AutoKeepNewestTimeCounter;
                bufferWindow.MoveNewest();
            }
        }

        private void BufferWindow_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsKeepNewest")
            {
                if (!bufferWindow.IsKeepNewest)
                {
                    //当前不是最新，开始倒计时
                    AutoKeepNewestTimeRemaining = AutoKeepNewestTimeCounter;
                    timer.Start();
                }
                else
                {
                    timer.Stop();
                }
            }
        }

        class GetMapperOfFlyData_Flow
        {
            int Index = 0;
            public GetMapperOfFlyData_Flow(int index)
            {
                Index = index;
            }
            public CartesianMapper<Lc_Flow> GetCartesianMapper()
            {
                return Mappers.Xy<Lc_Flow>()
                    .X(value => value.Time.Ticks)
                    .Y(value =>
                    {
                        if (Index < value.Items.Count())
                            return value.Items[Index].Flow;
                        else
                            return double.NaN;
                    });
            }
        }
        void BindingSeries()
        {
            //int item_series_cnt = Series.Count - 1;
            //if (item_series_cnt < weightSystemClient.Items.Count)
            //{
            //    for (int i = item_series_cnt; i < weightSystemClient.Items.Count; i++)
            //    {
            //        //< lvc:LineSeries Values = "{Binding Values}" LineSmoothness = "1" StrokeThickness = "3" Configuration = "{Binding Mapper_D}" ScalesYAt = "0"
            //        //            Stroke = "DarkBlue" Fill = "Transparent" PointGeometrySize = "0" />

            //        //    StackedAreaSeries series = new StackedAreaSeries();
            //        //series.Title = weightSystemClient.Items[i].Number;
            //        //series.Fill = weightSystemClient.Items[i].Color;
            //        //series.SetBinding(StackedAreaSeries.ValuesProperty, new Binding("Values") { Source = this });
            //        //series.Configuration = new GetMapperOfFlyData_Flow(i).GetCartesianMapper();
            //        //Series.Add(series);
            //        LineSeries series = new LineSeries();

            //        series.Title = weightSystemClient.Items[i].Number;
            //        series.Fill = new SolidColorBrush(Colors.Transparent);
            //        series.Stroke = weightSystemClient.Items[i].Color;
            //        series.LineSmoothness = 2;
            //        series.StrokeThickness = 3;
            //        series.Configuration = new GetMapperOfFlyData_Flow(i).GetCartesianMapper();
            //        series.PointGeometrySize = 0;
            //        //series.Visibility = Visibility.Hidden;
            //        series.SetBinding(LiveCharts.Wpf.Series.ValuesProperty, new Binding("Values") { Source = this });
            //        Series.Add(series);
            //    }
            //}
            //else if (item_series_cnt > weightSystemClient.Items.Count)
            //{
            //    int remove_cnt = item_series_cnt - weightSystemClient.Items.Count;
            //    //删除
            //    for(int i=0;i<remove_cnt;i++)
            //    {
            //        Series.RemoveAt(Series.Count - 1);
            //    }
            //}


            if (Items.Count != weightSystemService.Items.Count)
            {
                Items.Clear();
                for (int i = 0; i < weightSystemService.Items.Count; i++)
                {
                    FlowGraphVm_Item item = new FlowGraphVm_Item
                    {
                        Title = weightSystemService.Items[i].Number,
                        Fill = weightSystemService.Items[i].Color,
                        Visibility = Visibility.Hidden,
                        Index = i
                    };
                    Items.Add(item);
                }
                UpdateAxisYTarget();
            }

        }


        void ToValues()
        {
            if (bufferWindow.Record.Count <= ValuesCap)
            {
                Values.Clear();
                Values.AddRange(bufferWindow.Record);
            }
            else
            {
                Values.Clear();
                List<Lc_Flow> values = new List<Lc_Flow>();
                for (int i = 0; i < ValuesCap; i++)
                {
                    int idx = i * bufferWindow.Record.Count / ValuesCap;
                    values.Add(bufferWindow.Record[idx]);
                }
                Values.AddRange(values);
            }
            if (Values.Count() > 0)
            {
                BeginTime = Values.First().Time;
                EndTime = Values.Last().Time;

                UpdateAxisYTarget();
            }
            else
            {
                BeginTime = DateTime.MinValue;
                EndTime = DateTime.MinValue;
            }
        }

        void Update_y()
        {
            if (graphParams.IsAxisYRangeAuto)
            {
                MaxValue = double.NaN;
                MinValue = double.NaN;
            }
            else
            {
                MaxValue = AxisYTarget + graphParams.AxisYRange;
                MinValue = AxisYTarget - graphParams.AxisYRange;
            }
        }

        void UpdateAxisYTarget()
        {
            FObjBase.PollModule.Current.Poll_JustOnce(() =>
            {
                if (Values.Count() == 0)
                    return;

                Lc_Flow flows = Values.Last();

                for (int i = 0; i < flows.Items.Count(); i++)
                {
                    Items[i].Value = flows.Items[i].Flow;
                }
                TotalFlow = flows.Total;

                double target = TotalFlow;

                try
                {
                    FlowGraphVm_Item it = Items.First((i) => i.Visibility == Visibility.Visible);
                    if (it != null)
                        target = it.Value;
                }
                catch
                {

                }

                AxisYTarget = (int)target;
            }, this, MARKNO_UPDATE_Y);
        }



        public void Newest()
        {
            AutoKeepNewestTimeRemaining = AutoKeepNewestTimeCounter;
            bufferWindow.MoveNewest();
        }
        public void Next()
        {
            AutoKeepNewestTimeRemaining = AutoKeepNewestTimeCounter;
            bufferWindow.MoveNextPage();
        }
        public void Pre()
        {
            AutoKeepNewestTimeRemaining = AutoKeepNewestTimeCounter;
            bufferWindow.MovePrePage();
        }

        public void SelectAll()
        {
            foreach (FlowGraphVm_Item i in Items)
            {
                i.Visibility = Visibility.Hidden;
            }
            Stroke = new SolidColorBrush(Colors.Black);
            Mapper = ConfigurationAll;

            UpdateAxisYTarget();
        }
        public void SelectABC(FlowGraphVm_Item item)
        {
            foreach (FlowGraphVm_Item i in Items)
            {
                if (i != item)
                {
                    i.Visibility = Visibility.Hidden;
                }
                else
                {
                    i.Visibility = Visibility.Visible;
                }
            }
            Stroke = item.Fill;
            Mapper = new GetMapperOfFlyData_Flow(item.Index).GetCartesianMapper();

            UpdateAxisYTarget();
        }
    }

    public class FlowGraphVm_Item : INotifyPropertyChanged
    {

        public string Title { get; set; }
        /// <summary>
        /// 颜色
        /// </summary>
        public Brush Fill { get; set; }
        /// <summary>
        /// 显示
        /// </summary>
        public Visibility Visibility { get; set; }
        /// <summary>
        /// 值
        /// </summary>
        public double Value { get; set; }

        public int Index { get; set; }
        public event PropertyChangedEventHandler PropertyChanged;
    }
    public class FlowGraphParams : INotifyPropertyChanged
    {
        static FlowGraphParams current = null;
        public static FlowGraphParams Current
        {
            get
            {
                if (current == null)
                {
                    current = new FlowGraphParams();
                    current.Load();
                }
                return current;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// 窗口显示数据总量
        /// </summary>
        public int WindowSize { get; set; } = 1000;

        /// <summary>
        /// 折径，进风-出风, Y轴 自动设置 MaxValue， MinValue, 
        /// </summary>
        public bool IsAxisYRangeAuto { get; set; } = true;
        /// <summary>
        /// 折径，Y轴显示范围， 中间值一定是 FilmWidthSet, 单位mm
        /// MaxValue = FilmWidthSet+FilmWidthAxisYRange
        /// MinValue = FilmWidthSet-FilmWidthAxisYRange
        /// </summary>
        public int AxisYRange { get; set; } = 20;

        string FilePath;
        public FlowGraphParams() {
            string path = System.IO.Path.Combine(FlyLayout.BasePath, "flowgraph.json");
            FilePath = path;
        }
        /// <summary>
        /// 保存
        /// </summary>
        /// <returns></returns>
        public bool Save()
        {
            try
            {
                string json = Newtonsoft.Json.JsonConvert.SerializeObject(this);

                File.WriteAllText(FilePath, json);
            }
            catch
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// 读取
        /// </summary>
        /// <returns></returns>
        bool Load()
        {
            try
            {
                string json = File.ReadAllText(FilePath);
                Newtonsoft.Json.JsonConvert.PopulateObject(json, this);
                return true;
            }
            catch
            {
                return false;
            }
        }
    }

    public class UiModule2_FlowGraph : MultiLayout.UiModule.IUiModule2
    {
        /// <summary>
        /// 控件标题
        /// 它的值取决于culture
        /// </summary>
        public string Title=> "称重流量";

        public ComponentType Type => ComponentType.Graph;
        public bool IsUnique => false;

        /// <summary>
        /// 控件
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetComponent(int id, IUnityContainer container)
        {
            FlowGraph graph = new FlowGraph();
            container.BuildUp(graph);
            return graph;
        }

        /// <summary>
        /// 控件缩略图，用于编辑界面时，大致看看
        /// 创建时，需要给它唯一ID，让加载自己的数据
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public FrameworkElement GetThumbnail()
        {
            return new System.Windows.Controls.Grid();
        }


        /// <summary>
        /// 给出全部控件ID, 控件自行删除没有的参数
        /// </summary>
        /// <param name="IDs"></param>
        public void MatchParam(int[] IDs)
        {
        }
    }
}
